﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;

namespace _4200Prototyper
{
    public partial class Form1 : Form
    {
        //Define serialport here so it is public throughout the rest of the code
        SerialPort serialPort = new SerialPort();
        
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //Start up the timer
            timer.Enabled = true;
            timer.Interval = Convert.ToInt32(txtInterval.Text);

            //Make the output text readonly but set the colours so that it looks normal (readonly looks ugly!)
            txtOutput.ReadOnly = true;
            txtOutput.ForeColor = Color.Black;
            txtOutput.BackColor = Color.White;

            //Set up serialport stuff
            serialPort.Parity = Parity.None;
            serialPort.DataBits = 8;
            serialPort.DataReceived += new SerialDataReceivedEventHandler(DataReceivedHandler);
        }

        //Allow the serialport to continuously read input from the arduino sketch
        private void DataReceivedHandler(object sender, SerialDataReceivedEventArgs e)
        {
            SerialPort sp = (SerialPort)sender;
            string indata = sp.ReadExisting();
            txtInput.AppendText(indata);
        }

        //Actually send a command to the serialport
        void SendCommand(TextBox textBox)
        {
            try
            {
                serialPort.Write(textBox.Text);
                txtOutput.AppendText(textBox.Text + Environment.NewLine);
            }
            catch
            {
                btnStopTimer.Enabled = false;
                btnStartTimer.Enabled = true;
                timer.Enabled = false;
                MessageBox.Show("There was a problem writing to the serial port. Timer has been stopped.", "ERROR");

            }

        }

        //Hold command - just changes the button text really
        void HoldCommand(TextBox textBox, Button button)
        {
            if (button.Text == "Stop Hold")
            {
                button.Text = "Hold";
                button.ForeColor = Color.Black;
            }
            else
            {
                button.Text = "Stop Hold";
                button.ForeColor = Color.Red;
            }
        }

        //Check for holds everytime timer ticks
        private void timer_Tick(object sender, EventArgs e)
        {
            if (btnHold1.Text == "Stop Hold")
                SendCommand(txt1);
            else if (btnHold2.Text == "Stop Hold")
                SendCommand(txt2);
            else if (btnHold3.Text == "Stop Hold")
                SendCommand(txt3);
            else if (btnHold4.Text == "Stop Hold")
                SendCommand(txt4);
            else if (btnHold5.Text == "Stop Hold")
                SendCommand(txt5);
            else if (btnHold6.Text == "Stop Hold")
                SendCommand(txt6);
            else if (btnHold7.Text == "Stop Hold")
                SendCommand(txt7);
            else if (btnHold8.Text == "Stop Hold")
                SendCommand(txt8);
            else if (btnHold9.Text == "Stop Hold")
                SendCommand(txt9);
            else if (btnHold10.Text == "Stop Hold")
                SendCommand(txt10);
            else if (btnHold11.Text == "Stop Hold")
                SendCommand(txt11);
            else if (btnHold12.Text == "Stop Hold")
                SendCommand(txt12);
            else if (btnHold13.Text == "Stop Hold")
                SendCommand(txt13);
            else if (btnHold14.Text == "Stop Hold")
                SendCommand(txt14);
            else if (btnHold15.Text == "Stop Hold")
                SendCommand(txt15);
            else if (btnHold16.Text == "Stop Hold")
                SendCommand(txt16);
        }

        //Connect to the serialport
        private void btnConnect_Click(object sender, EventArgs e)
        {
            try
            {
                serialPort.BaudRate = 9600;
                serialPort.PortName = txtPort.Text;

                serialPort.Open();

                txtOutput.AppendText("SERIAL PORT OPENED" + Environment.NewLine);

                btnDisconnect.Enabled = true;
                btnConnect.Enabled = false;
            }
            catch
            {
                MessageBox.Show("There was an error connecting to the serial port", "Error");
            }
        }

        //Disconnect from the serialport
        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            serialPort.Close();

            btnConnect.Enabled = true;
            btnDisconnect.Enabled = false;

            txtOutput.AppendText("SERIAL PORT CLOSED" + Environment.NewLine);
        }

        //Update the timer interval to slow down/speed up serial port output for Hold buttons
        private void btnUpdateInterval_Click(object sender, EventArgs e)
        {
            int parsedValue;
            if (!int.TryParse(txtInterval.Text, out parsedValue))
            {
                MessageBox.Show("Interval has to be an integer value.", "Error");
                return;
            }

            timer.Interval = Convert.ToInt32(txtInterval.Text);
        }

        //Clear the output textbox and reset the Lines text
        private void btnClear_Click(object sender, EventArgs e)
        {
            lblLines.Text = "0";
            txtOutput.Text = "";
            txtOutput.AppendText("CLEARED" + Environment.NewLine);
            txtInput.Text = "";
            txtInput.AppendText("CLEARED" + Environment.NewLine);
        }

        //Update number of lines in output textbox
        private void txtOutput_TextChanged(object sender, EventArgs e)
        {
            var lines = txtOutput.Lines.Count();
            lines -= String.IsNullOrWhiteSpace(txtOutput.Lines.Last()) ? 1 : 0;
            lblLines.Text = Convert.ToString(lines);
        }

        //Update number of lines in input textbox
        private void txtInput_TextChanged(object sender, EventArgs e)
        {
            var lines = txtInput.Lines.Count();
            lines -= String.IsNullOrWhiteSpace(txtInput.Lines.Last()) ? 1 : 0;
            lblInputLines.Text = Convert.ToString(lines);
        }

        //Stop the timer manually
        private void btnStopTimer_Click(object sender, EventArgs e)
        {
            timer.Enabled = false;
            btnStopTimer.Enabled = false;
            btnStartTimer.Enabled = true;
        }

        //Start the timer manually
        private void btnStartTimer_Click(object sender, EventArgs e)
        {
            timer.Enabled = true;
            btnStartTimer.Enabled = false;
            btnStopTimer.Enabled = true;
        }

        //Hold Buttons
        private void btnHold1_Click(object sender, EventArgs e)
        {
            HoldCommand(txt1, btnHold1);
        }

        private void btnHold2_Click(object sender, EventArgs e)
        {
            HoldCommand(txt2, btnHold2);
        }

        private void btnHold3_Click(object sender, EventArgs e)
        {
            HoldCommand(txt3, btnHold3);
        }

        private void btnHold4_Click(object sender, EventArgs e)
        {
            HoldCommand(txt4, btnHold4);
        }

        private void btnHold5_Click(object sender, EventArgs e)
        {
            HoldCommand(txt5, btnHold5);
        }

        private void btnHold6_Click(object sender, EventArgs e)
        {
            HoldCommand(txt6, btnHold6);
        }

        private void btnHold7_Click(object sender, EventArgs e)
        {
            HoldCommand(txt7, btnHold7);
        }

        private void btnHold8_Click(object sender, EventArgs e)
        {
            HoldCommand(txt8, btnHold8);
        }

        private void btnHold9_Click(object sender, EventArgs e)
        {
            HoldCommand(txt9, btnHold9);
        }

        private void btnHold10_Click(object sender, EventArgs e)
        {
            HoldCommand(txt10, btnHold10);
        }

        private void btnHold11_Click(object sender, EventArgs e)
        {
            HoldCommand(txt11, btnHold11);
        }

        private void btnHold12_Click(object sender, EventArgs e)
        {
            HoldCommand(txt12, btnHold12);
        }

        private void btnHold13_Click(object sender, EventArgs e)
        {
            HoldCommand(txt13, btnHold13);
        }

        private void btnHold14_Click(object sender, EventArgs e)
        {
            HoldCommand(txt14, btnHold14);
        }

        private void btnHold15_Click(object sender, EventArgs e)
        {
            HoldCommand(txt15, btnHold15);
        }

        private void btnHold16_Click(object sender, EventArgs e)
        {
            HoldCommand(txt16, btnHold16);
        }

        //Sender buttons
        private void btnSend1_Click(object sender, EventArgs e)
        {
            SendCommand(txt1);
        }

        private void btnSend2_Click(object sender, EventArgs e)
        {
            SendCommand(txt2);
        }

        private void btnSend3_Click(object sender, EventArgs e)
        {
            SendCommand(txt3);
        }

        private void btnSend4_Click(object sender, EventArgs e)
        {
            SendCommand(txt4);
        }

        private void btnSend5_Click(object sender, EventArgs e)
        {
            SendCommand(txt5);
        }

        private void btnSend6_Click(object sender, EventArgs e)
        {
            SendCommand(txt6);
        }

        private void btnSend7_Click(object sender, EventArgs e)
        {
            SendCommand(txt7);
        }

        private void btnSend8_Click(object sender, EventArgs e)
        {
            SendCommand(txt8);
        }

        private void btnSend9_Click(object sender, EventArgs e)
        {
            SendCommand(txt9);
        }

        private void btnSend10_Click(object sender, EventArgs e)
        {
            SendCommand(txt10);
        }

        private void btnSend11_Click(object sender, EventArgs e)
        {
            SendCommand(txt11);
        }

        private void btnSend12_Click(object sender, EventArgs e)
        {
            SendCommand(txt12);
        }

        private void btnSend13_Click(object sender, EventArgs e)
        {
            SendCommand(txt13);
        }

        private void btnSend14_Click(object sender, EventArgs e)
        {
            SendCommand(txt14);
        }

        private void btnSend15_Click(object sender, EventArgs e)
        {
            SendCommand(txt15);
        }

        private void btnSend16_Click(object sender, EventArgs e)
        {
            SendCommand(txt16);      
        }
    }
}
